@@ -16,63 +16,90 @@
 #define _INCLUDE_CONF_H
 
 /* 
-    more parameters may be added later.
+	more parameters may be added later.
  */
 struct config {
-        int width;
-        int height;
+	int width;
+	int height;
 	int quality;
+	int rotate_deg;
 	int max_changes;
+	int threshold_tune;
 	int always_diff;
 	int new_img;
 	int motion_img;
-	int snap_overwrite;
+	int output_all;
 	int gap;
 	int maxmpegtime;
+	int snapshot_interval;
 	int locate;
 	int input;
 	int norm;
 	int frame_limit;
 	int mpeg;
-        int quiet;
+	int quiet;
 	int ppm;
 	int noise;
+	int noise_tune;
 	int mingap;
 	int lightswitch;
 	int jpg_cleanup;
 	int nightcomp;
 	int adjustrate;
-	int realmotion;
-	int oldlayout;
+	int oldlayout; /* Only used for Berkeley mpeg_encode now */
 	int low_cpu;
 	int nochild;
 	int autobright;
-	int roundrobing_frames;
-	int roundrobing_skip;
+	int roundrobin_frames;
+	int roundrobin_skip;
+	int pre_capture;
 	int post_capture;
 	int switchfilter;
 	int ffmpeg_cap_new;
 	int ffmpeg_cap_motion;
 	int ffmpeg_bps;
+	int ffmpeg_vbr;
+	char *ffmpeg_video_codec;
 	int webcam_port;
 	int webcam_quality;
 	int webcam_motion;
 	int webcam_maxrate;
+	int webcam_localhost;
+	int webcam_limit;
+	int control_port;
+	int control_localhost;
 	int frequency;
-	int timelaps;
-	char *device;
+	int timelapse;
+	char *timelapse_mode; 
+	int predict_threshold;
+	int predict_enable;
+#ifdef __freebsd__
+	char *tuner_device;
+#endif
+	char *video_device;
 	char *vidpipe;
 	char *mail_address;
 	char *sms_nr;
 	char *filepath;
+	char *jpegpath;
+	char *mpegpath;
+	char *snappath;
+	char *timepath;
+	char *predictpath;
 	char *externcommand;
 	char *mask_file;
+	int sql_log_image;
+	int sql_log_snapshot;
+	int sql_log_mpeg;
+	int sql_log_timelapse;
+	int sql_log_prediction;
 	char *mysql_db;
 	char *mysql_host;
 	char *mysql_user;
 	char *mysql_password;
 	char *onsavecommand;
 	char *onmpegcommand;
+	char *onffmpegclosecommand;
 	char *motionvidpipe;
 	char *netcam_url;
 	char *netcam_userpass;
@@ -82,9 +109,12 @@ struct config {
 	char *pgsql_password;
 	int pgsql_port;
 	char *mpeg_encode_bin;
-	int drawtext_changes;
-	int drawtext_shots;
-	char *drawtext_user;
+	int text_changes;
+	char *text_left;
+	char *text_right;
+	char *despeckle;
+	int minimum_motion_frames;
+	int debug_parameter;
 	int argc;
 	char **argv;
 };
@@ -95,19 +125,28 @@ extern struct config conf;
  * typedef for a param copy function. 
  */
 typedef struct context ** (* conf_copy_func)(struct context **, char *, int) ;
+typedef char *(* conf_print_func)(struct context **, char **, int, int);
 
 /**
  * description for parameters in the config file
  */
 typedef struct {
-  char * param_name;	/* name for this parameter             */
-  int conf_value;	/* pointer to a field in struct context */
-  conf_copy_func  copy;	/* a function to set the value in 'config'*/
+	char * param_name;	/* name for this parameter             */
+	char * param_help;	/* short explanation for parameter */
+	int conf_value;	/* pointer to a field in struct context */
+	conf_copy_func  copy;	/* a function to set the value in 'config' */
+	conf_print_func print;	/* a function to output the value to a file */
 } config_param; 
 
 
-//extern struct config config;
+extern config_param config_params[];
+
 struct context **conf_load (struct context **);
-//void conf_process (FILE *fp);
+struct context **conf_cmdparse(struct context **, char *, char *);
+char *config_type(config_param *);
+void conf_print (struct context **);
+void malloc_strings (struct context *);
+char *mystrdup(char *);
+char *mystrcpy(char *, char *);
 
 #endif /* _INCLUDE_CONF_H */
