@@ -6,14 +6,15 @@
 #define EVENT_FIRSTMOTION	4
 #define EVENT_ENDMOTION		8
 #define EVENT_STOP		16
-#define EVENT_TIMELAPS		32
-#define EVENT_TIMELAPSEND	64
+#define EVENT_TIMELAPSE		32
+#define EVENT_TIMELAPSEEND	64
 
 #define EVENT_IMAGE_DETECTED	256
 #define EVENT_IMAGEM_DETECTED	512
 #define EVENT_IMAGE_SNAPSHOT	1024
 #define EVENT_IMAGE		2048
-#define EVENT_IMAGEM		8192	
+#define EVENT_IMAGEM		8192
+#define EVENT_FILECLOSE		16384
 
 #define EVENT_DEBUG		65536
 #define EVENT_CRITICAL		131072
