@@ -13,46 +13,6 @@
 
 #include <jpeglib.h>
 
-void put_jpeg_bgr24 (FILE *picture, char *image, int width, int height, int quality)
-{
-	int y, x, line_width;
-	JSAMPROW row_ptr[1];
-	struct jpeg_compress_struct cjpeg;
-	struct jpeg_error_mgr jerr;
-	unsigned char *line;
-
-	line=malloc(width *3);
-	if (!line)
-		return;
-	cjpeg.err = jpeg_std_error(&jerr);
-	jpeg_create_compress (&cjpeg);
-	cjpeg.image_width = width;
-	cjpeg.image_height = height;
-	cjpeg.input_components = 3;
-	cjpeg.in_color_space = JCS_RGB;
-	jpeg_set_defaults (&cjpeg);
-
-	jpeg_set_quality (&cjpeg, quality, TRUE);
-	cjpeg.dct_method = JDCT_FASTEST;
-	jpeg_stdio_dest (&cjpeg, picture);
-
-	jpeg_start_compress (&cjpeg, TRUE);
-
-	row_ptr[0] = line;
-	line_width = width * 3;
-	for (y=0; y<height; y++) {
-		for (x=0; x<line_width; x+=3) {
-			line[x]   = image[x+2];
-			line[x+1] = image[x+1];
-			line[x+2] = image[x];
-		}
-		jpeg_write_scanlines (&cjpeg, row_ptr, 1);
-		image += line_width;
-	}
-	jpeg_finish_compress (&cjpeg);
-	jpeg_destroy_compress (&cjpeg);
-	free(line);
-}
 
 // twice as fast as RGB jpeg 
 void put_jpeg_yuv420p (FILE *fp, unsigned char *image, int width, int height, int quality)
@@ -92,7 +52,7 @@ void put_jpeg_yuv420p (FILE *fp, unsigned char *image, int width, int height, in
 
 	jpeg_stdio_dest (&cinfo, fp);  	  // data written to file
 	jpeg_start_compress (&cinfo, TRUE);
-  
+
 	for (j=0;j<height;j+=16) {
 		for (i=0;i<16;i++) {
 			y[i] = image + width*(i+j);
@@ -102,7 +62,7 @@ void put_jpeg_yuv420p (FILE *fp, unsigned char *image, int width, int height, in
 			}
 		}
 		jpeg_write_raw_data (&cinfo, data, 16);
-  	}
+	}
 
 	jpeg_finish_compress (&cinfo);
 	jpeg_destroy_compress (&cinfo);
@@ -142,6 +102,11 @@ void put_jpeg_grey (FILE *picture, char *image, int width, int height, int quali
 void put_ppm_bgr24 (FILE *picture, char *image, int width, int height)
 {
 	int x, y;
+	unsigned char *l=image;
+	unsigned char *u=image+width*height;
+	unsigned char *v=u+(width*height)/4;
+	int r, g, b;
+	unsigned char rc, gc, bc;
 
 	/*	ppm header
 	 *	width height
@@ -152,10 +117,41 @@ void put_ppm_bgr24 (FILE *picture, char *image, int width, int height)
 	fprintf(picture, "%d\n", 255);
 	for (y=0; y<height; y++) {
 		for (x=0; x<width; x++) {
+			r = 76283*(((int)*l)-16)+104595*(((int)*u)-128);
+			g = 76283*(((int)*l)-16)- 53281*(((int)*u)-128)-25625*(((int)*v)-128);
+			b = 76283*(((int)*l)-16)+132252*(((int)*v)-128);
+			r = r>>16;
+			g = g>>16;
+			b = b>>16;
+			if (r<0)
+				r=0;
+			else if (r>255)
+				r=255;
+			if (g<0)
+				g=0;
+			else if (g>255)
+				g=255;
+			if (b<0)
+				b=0;
+			else if (b>255)
+				b=255;
+			rc=r;
+			gc=g;
+			bc=b;
+
+			l++;
+			if (x&1) {
+				u++;
+				v++;
+			}
 			/* ppm is rgb not bgr */
-			fwrite(&image[y*width*3+x*3+2], 1, 1, picture);
-			fwrite(&image[y*width*3+x*3+1], 1, 1, picture);
-			fwrite(&image[y*width*3+x*3+0], 1, 1, picture);
+			fwrite(&bc, 1, 1, picture);
+			fwrite(&gc, 1, 1, picture);
+			fwrite(&rc, 1, 1, picture);
+		}
+		if (y&1) {
+			u-=width/2;
+			v-=width/2;
 		}
 	}
 }
@@ -165,10 +161,8 @@ void put_picture_fd (struct context *cnt, FILE *picture, char *image, int qualit
 	if (cnt->conf.ppm) {
 		put_ppm_bgr24(picture, image, cnt->conf.width, cnt->conf.height);
 	} else {
+//		printf("type: %d %d\n", cnt->imgs.type, VIDEO_PALETTE_YUV420P);
 		switch (cnt->imgs.type) {
-			case VIDEO_PALETTE_RGB24:
-				put_jpeg_bgr24(picture, image, cnt->conf.width, cnt->conf.height, quality);
-				break;
 			case VIDEO_PALETTE_YUV420P:
 				put_jpeg_yuv420p(picture, image, cnt->conf.width, cnt->conf.height, quality);
 				break;
@@ -179,20 +173,25 @@ void put_picture_fd (struct context *cnt, FILE *picture, char *image, int qualit
 	}
 }
 
-void put_picture (struct context *cnt, char *file, char *image)
+void put_picture (struct context *cnt, char *file, char *image, int ftype)
 {
 	FILE *picture;
 
-	picture=fopen(file, "w");
+	picture=myfopen(file, "w");
 	if (!picture) {
-		perror("can't write picture!");
-		return;
+		/* Report to syslog - suggest solution if the problem is access rights to target dir */
+		if (errno ==  EACCES){
+			syslog(LOG_ERR, "can't write picture to file %s - check access rights to target directory: %m", file);
+			cnt->finish = 1;
+			return;
+		} else {
+			/* If target dir is temporarily unavailable we may survive */
+			syslog(LOG_ERR, "can't write picture to file %s: %m", file);
+			return;
+		}
 	}
+
 	put_picture_fd(cnt, picture, image, cnt->conf.quality);
-	if (cnt->conf.ppm) {
-		event(EVENT_FILECREATE, cnt, file, "image/ppm", cnt->currenttime);
-	} else {
-		event(EVENT_FILECREATE, cnt, file, "image/jpg", cnt->currenttime);
-	}
+	event(EVENT_FILECREATE, cnt, file, (void *)ftype,NULL);
 	fclose(picture);
 }
