@@ -8,6 +8,6 @@
 #include "motion.h"
 
 void put_picture_fd (struct context *, FILE *, char *, int);
-void put_picture (struct context *, char *, char *);
+void put_picture (struct context *, char *, char *, int );
 
 #endif /* _INCLUDE_PICTURE_H_ */
