@@ -1,4 +1,4 @@
-/*	motion.h
+/*	video.h
  *
  *	Include file for video.c
  *      Copyright 2000 by Jeroen Vreeken (pe1rxq@amsat.org)
@@ -10,7 +10,10 @@
 #ifndef _INCLUDE_VIDEO_H
 #define _INCLUDE_VIDEO_H
 
+#define _LINUX_TIME_H 1
+#ifndef WITHOUT_V4L
 #include <linux/videodev.h>
+#endif
 
 /* video4linux stuff */
 #define NORM_DEFAULT    0
@@ -28,11 +31,11 @@
 
 struct video_dev {
 	int fd;
-	char *device;
+	char *video_device;
 	int input;
 	int width;
 	int height;
-	long freq;
+	unsigned long freq;
 
 	pthread_mutex_t mutex;
 	pthread_mutexattr_t attr;
@@ -40,7 +43,7 @@ struct video_dev {
 	int frames;
 	
 	/* Device type specific stuff: */
-	
+#ifndef WITHOUT_V4L	
 	/* v4l */
 	int v4l_fmt;
 	int v4l_read_img;
@@ -48,23 +51,25 @@ struct video_dev {
 	int v4l_curbuffer;
 	int v4l_maxbuffer;
 	int v4l_bufsize;
+#endif
 };
 
 /* video functions, video.c */
-void vid_init(void);
-int vid_start (struct context *, int width, int height, int input, int norm);
+int vid_start (struct context *, int width, int height, int input, int norm, unsigned long frequency);
 char *vid_next (struct context *, int dev, char *map, int width, int height);
+#ifndef WITHOUT_V4L
+void vid_init(void);
 int vid_startpipe (char *devname, int width, int height, int);
 int vid_putpipe (int dev, char *image, int);
-void vid_autobright (struct config *, int dev, char *image, int height, int width);
-char *vid_keepalive(struct context *, struct images *imgs, int dev, int pipe, int mpipe);
+void vid_autobright (struct config *, int dev, char *image, int width, int height);
 void vid_close(void);
 void vid_cleanup(void);
+#endif
 
 #ifdef HAVE_CURL
 /* Network camera functions, netcam.c */
 int netcam_start (struct context *);
-unsigned char *netcam_next (struct context *);
+unsigned char *netcam_next (struct context *, char *);
 #endif /* HAVE_CURL */
 
 #endif /* _INCLUDE_VIDEO_H */
